namespace L {
/**
 * Exponentiation @f$x^p@f$.
 * 
 * @tparam x Base
 * @tparam Res_t Result type
 * @param p Non-negative exponent
 * @return @f$x^p@f$
 */
template<int x, typename Res_t>
Res_t exp(unsigned int p) {
	if (p == 0) {
		return 1;
	}

	Res_t y = x,
	      r = 1;
	while (p != 0) {
		if (p & 1) {
			r *= y;
		}

		p >>= 1;
		y *= y;
	}

	return r;
}

/**
 * Exponentiation @f$x^p@f$.
 * Default value type <code>long long</code>.
 * 
 * @tparam x Base
 * @param p Non-negative exponent
 * @return @f$x^p@f$
 */
template<int x>
inline long long exp(unsigned int p) {
	return exp<x, long long>(p);
}

/**
 * Exponentiation @f$2^p@f$.
 * 
 * @param p Non-negative exponent
 * @return @f$2^p@f$
 */
template<>
inline long long exp<2, long long>(unsigned int p) {
	return 1LL << p;
}
}